/**
 * Copyright (C) 2012 Michael Oestergaard Pedersen.
 * Distributed under the GNU GPL v3. For full terms see the file LICENSE.txt.
 *
 * This file is part of VerifiedQR.
 *
 * VerifiedQR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VerifiedQR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VerifiedQR.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.io.IOException;
import java.util.Vector;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import dk.bytopia.crypto.signature.bls.BLS;
import dk.bytopia.verifiedqr.VerifiedQR;

public class VerifiedQRTest {

    public static final String curve = "curves/curve_d160.properties";
    public static final String publicKey = "key.public";
    public static final String secretKey = "key.secret";
    public static final String output = "qr.png";

    public static void main(String[] args) throws IllegalArgumentException, InvalidKeyException {
	// Initialise the BLS signature scheme, generate keys and store them
	BLS bls = new BLS(curve);
	bls.keygen();
	bls.storePublicKey(publicKey);
	bls.storeSecretKey(secretKey);

	// Initialise a new instance of the BLS signature scheme and load the keys
	bls = new BLS(curve);
	bls.loadPublicKey(publicKey);
	bls.loadSecretKey(secretKey);

	// Create a vector with the values to store in the QR code
	Vector<String> vec = new Vector<String>();
	vec.add("John Doe");
	vec.add("123-4567-890");

	// Initialise the QR generator
	VerifiedQR qr = new VerifiedQR();

	// Encode and sign the data using BLS
	String strData = qr.encodeAndSign(bls, vec);

	// Generate the QR code as a 200x200 PNG file
	try {
	    qr.generate(output, strData, 200, 200);
	} catch (IOException e) {
	    System.out.println("Error generating QR file: " + e.getMessage());
	}

	// Show information about the data
	System.out.println("Data: " + strData);
	System.out.println("Size: " + strData.length());
	System.out.println();

	// Initialise a new instance of the BLS signature scheme and load the public key
	bls = new BLS(curve);
	bls.loadPublicKey(publicKey);

	// Decode the data stored in the QR code and verify the signature
	try {
	    vec = qr.decodeAndVerify(bls, strData);
	    if (vec != null)
		System.out.println("Verified data: " + vec);
	} catch (SignatureException e) {
	    System.out.println(e.getMessage());
	}
    }
}
